#include "BensleyTones.h"
#include <TimerOne.h>
#include <stdlib.h>

// Floppy drive music take 1!

//#define DriveSelect 5 // IDE 14 : Not Used : Grounded 
//#define f1Direction 4 // IDE 18, LOW = move forward, HIGH = move backward
//#define f1HeadStep 5 // IDE 20
//#define f2Direction 2
//#define f2HeadStep 3

int aryPinDirection[] = {2, 4, 6};
int aryPinHeadStep[] = {3, 5, 7};

/*
* BMP = 120
* Thats 1 note every 500ms
* half note = 250ms
* quater note = 125ms
* 
*/
const int bmp = 120;
const int beat = (60000/bmp);
const long scale = 440000;
const int timerDelay = 70;
const float timeMinus = 70;

        /*
        beat / length = note note length in ms
        500ms / 2 = 250ms
        
        1000ms / frequency = 1 steps every X ms
        1000/100 = 10ms
        
        note length in ms / steps every X ms = number of steps
        250ms / 10ms = 25
        
        step delay = 1 step every X ms / 2
        10 / 2 = 5ms

        RESTS: REST_WHOLE = 4
        beat / length = note note length in ms
        500ms / 2 = 250ms
        
        1000ms / frequency = 1 steps every X ms
        1000/4 = 250ms
        
        note length in ms / steps every X ms = number of steps
        250ms / 250ms = 1
        
        step delay = 1 step every X ms / 2
        250 / 2 = 125
        
        RESTS: REST_WHOLE = 4
        beat / length = note note length in ms
        1000ms / 1 = 1000ms
        
        1000ms / frequency = 1 steps every X ms
        1000/4 = 250ms
        
        note length in ms / steps every X ms = number of steps
        1000ms / 250ms = 4
        
        step delay = 1 step every X ms / 2
        250 / 2 = 125
        
        RESTS: REST = 2
        beat / length = note note length in ms
        1000ms / 2 = 500ms
        
        1000ms / frequency = 1 steps every X ms
        1000/2 = 500ms
        
        note length in ms / steps every X ms = number of steps
        500ms / 500ms = 1
        
        step delay = 1 step every X ms / 2
        500 / 2 = 250
        */

int aryNotes0[] = {  // Drive 1 High - This has to be a float if using any of the REST_x notes
  // Bars 1-4
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,
  NOTE_F2,NOTE_CS2,NOTE_F2,NOTE_E2,NOTE_C1,NOTE_E2,
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_B2,
  NOTE_A2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_A2,REST,
  // Bars 5-8
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_D2,REST,NOTE_A2,REST,NOTE_D2,REST,NOTE_A2,REST,
  // Bars 9 to 12
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST,
  NOTE_D3,NOTE_AS2,NOTE_D3,NOTE_CS3,NOTE_AS2,NOTE_CS3,
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST,
  NOTE_D3,NOTE_AS2,NOTE_D3,NOTE_CS3,REST,
  // Bars 13 to 16
  NOTE_FS2,NOTE_CS3,NOTE_FS2,NOTE_AS2,
  NOTE_D2,NOTE_AS2,NOTE_FS2,NOTE_AS2,
  NOTE_FS2,NOTE_CS3,NOTE_FS2,NOTE_AS2,
  NOTE_D2,NOTE_AS2,NOTE_FS2,NOTE_AS2
};

// The note duration is a faction of a whole beat so we can have crotchets and quavers etc
// 1 / 2 = half beat, 1 / 4 = quater beat
// this has to be a float if we want to double up, as its used in a division,
// 1 / 0.5 = 2 for 2 beats long
int aryDuration0[] = { // Drive 1 High
  // Bars 1-4
  2,2,2,2,2,2,1,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,    // 27th Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,  //  49th Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,1,1,1,        //  76th Note
  // Bars 13-16
  1,1,1,1,
  1,1,1,1,
  1,1,1,1,
  1,1,1,1          //  92nd Note
};

int aryNotes1[] = { // Drive 1 Low
  // Bars 1-4
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,
  NOTE_F1,NOTE_CS1,NOTE_F1,NOTE_E1,NOTE_C1,NOTE_E1,
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_B1,
  NOTE_A1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_A1,REST,
  // Bars 5-8
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_D1,REST,NOTE_A1,REST,NOTE_D1,REST,NOTE_A1,REST,
  // Bars 9-12
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST,
  NOTE_D2,NOTE_AS1,NOTE_D2,NOTE_CS2,NOTE_AS1,NOTE_CS2,
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST,
  NOTE_D2,NOTE_AS1,NOTE_D2,NOTE_CS2,REST,
  // Bars 13-16
  NOTE_FS1,NOTE_CS2,NOTE_FS1,NOTE_AS1,
  NOTE_D1,NOTE_AS1,NOTE_FS1,NOTE_AS1,
  NOTE_FS1,NOTE_CS2,NOTE_FS1,NOTE_AS1,
  NOTE_D1,NOTE_AS1,NOTE_FS1,NOTE_AS1
};

int aryDuration1[] = { // Drive 1 Low
  // Bars 1-4
  2,2,2,2,2,2,1,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,    // 27th Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,  //  49th Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,1,2,2,1,
  2,2,2,2,2,2,2,2,
  2,2,1,1,1,        //  76th Note
  // Bars 13-16
  1,1,1,1,
  1,1,1,1,
  1,1,1,1,
  1,1,1,1          //  92nd Note
};

int aryNotes2[] = { // Drive 2 High
  // Bars 1-4
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_B1,REST,NOTE_FS2,REST,NOTE_B1,REST,NOTE_FS2,REST,
  NOTE_D2,REST,NOTE_A2,REST,NOTE_D2,REST,NOTE_A2,REST,
  // Bars 5-8
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,REST,
  NOTE_F2,NOTE_CS2,NOTE_F2,REST,NOTE_E2,NOTE_B1,NOTE_E2,REST,
  NOTE_B1,NOTE_CS2,NOTE_D2,NOTE_E2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_B2,
  NOTE_A2,NOTE_FS2,NOTE_D2,NOTE_FS2,NOTE_A2,REST,
  // Bars 9-12
  NOTE_FS2,REST,NOTE_CS3,REST,NOTE_FS2,REST,NOTE_AS2,REST,
  NOTE_D2,REST,NOTE_AS2,REST,NOTE_FS2,REST,NOTE_AS2,REST,
  NOTE_FS2,REST,NOTE_CS3,REST,NOTE_FS2,REST,NOTE_AS2,REST,
  NOTE_D2,REST,NOTE_AS2,REST,NOTE_FS2,REST,NOTE_AS2,REST,
  // Bars 13-16
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST,
  NOTE_D3,NOTE_AS2,NOTE_D3,REST,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST,
  NOTE_FS2,NOTE_GS2,NOTE_AS2,NOTE_B2,NOTE_CS3,NOTE_AS2,NOTE_CS3,REST,
  NOTE_D3,NOTE_AS2,NOTE_D3,REST,NOTE_CS3,REST,REST
};

int aryDuration2[] = { // Drive 2 High
  // Bars 1-4
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 32nd Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,        // 62nd Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 94th Note
  // Bars 13-16
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,1       // 124th Note
};

int aryNotes3[] = { // Drive 2 Low
  // Bars 1-4
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_B0,REST,NOTE_FS1,REST,NOTE_B0,REST,NOTE_FS1,REST,
  NOTE_D1,REST,NOTE_A1,REST,NOTE_D1,REST,NOTE_A1,REST,
  // Bars 5-8
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,REST,
  NOTE_F1,NOTE_CS1,NOTE_F1,REST,NOTE_E1,NOTE_B0,NOTE_E1,REST,
  NOTE_B0,NOTE_CS1,NOTE_D1,NOTE_E1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_B1,
  NOTE_A1,NOTE_FS1,NOTE_D1,NOTE_FS1,NOTE_A1,REST,
  // Bars 9-12
  NOTE_FS1,REST,NOTE_CS2,REST,NOTE_FS1,REST,NOTE_AS1,REST,
  NOTE_D1,REST,NOTE_AS1,REST,NOTE_FS1,REST,NOTE_AS1,REST,
  NOTE_FS1,REST,NOTE_CS2,REST,NOTE_FS1,REST,NOTE_AS1,REST,
  NOTE_D1,REST,NOTE_AS1,REST,NOTE_FS1,REST,NOTE_AS1,REST,
  // Bars 13-16
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST,
  NOTE_D2,NOTE_AS1,NOTE_D2,REST,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST,
  NOTE_FS1,NOTE_GS1,NOTE_AS1,NOTE_B1,NOTE_CS2,NOTE_AS1,NOTE_CS2,REST,
  NOTE_D2,NOTE_AS1,NOTE_D2,REST,NOTE_CS2,REST,REST
};

int aryDuration3[] = { // Drive 2 Low
  // Bars 1-4
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 32nd Note
  // Bars 5-8
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,1,1,        // 62nd Note
  // Bars 9-12
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,    // 94th Note
  // Bars 13-16
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,2,2,
  2,2,2,2,2,2,1       // 124th Note
};


const int driveCount = 2;
int driveIndex;
const int driveStart = 0;

int aryNoteIndex[3] = {0, 0, 0};
int aryNoteCount[3] = {102, 102, 124};
long aryNoteLength[3] = {0, 0, 0}; // Number of steps
long aryNoteDelay[3] = {0, 0, 0}; // Delay between steps
int aryHighLow[3] = {0, 0, 0};
float aryWaiting[3] = {0, 0, 0};
int aryBuffer[3] = {0, 0, 0}; // Less than a whole step at the end, ms of note buffer
int aryRunningScew[3] = {0, 0, 0};

long noteLengthInMs=0;
float freqOfSteps=0;
int remainder=0;
int scew=0;

void centerHead() {
  digitalWrite(aryPinHeadStep[driveIndex], HIGH);
  digitalWrite(aryPinDirection[driveIndex], HIGH);
  // Move header to start position from wherever it is, 80 steps at ~261 step/s
  for(int i=0;i<80;i++) {
    digitalWrite(aryPinHeadStep[driveIndex],LOW);
    delay(2);
    digitalWrite(aryPinHeadStep[driveIndex],HIGH); 
    delay(2);
  }
  delay(2);
  digitalWrite(aryPinDirection[driveIndex], LOW); // Change direction, and move then back 40 steps so we are centered (the step range is 80 wide)
  for(int i=0;i<40;i++) {
    digitalWrite(aryPinHeadStep[driveIndex],LOW);
    delay(2);
    digitalWrite(aryPinHeadStep[driveIndex],HIGH); 
    delay(2);
  }
}

void setupNote(){
  if(aryNoteIndex[driveIndex]<aryNoteCount[driveIndex]) {
    if(driveIndex==0) {
        noteLengthInMs = beat / aryDuration0[aryNoteIndex[driveIndex]];
        noteLengthInMs = noteLengthInMs * 1000;
        freqOfSteps = 1000 / aryNotes0[aryNoteIndex[driveIndex]];
        freqOfSteps = freqOfSteps * 1000;
        
        aryNoteLength[driveIndex] = noteLengthInMs / freqOfSteps;
        aryNoteDelay[driveIndex] = freqOfSteps / 2;
        
        remainder = noteLengthInMs - (aryNoteLength[driveIndex] * freqOfSteps);
        if(remainder > 0) {
          scew = remainder % 100;
          remainder = remainder - scew;
          aryRunningScew[driveIndex] += scew;
          
          if(aryRunningScew[driveIndex]>=100) {
            remainder += 100;
            aryRunningScew[driveIndex] -= 100;
          }
          
          aryBuffer[driveIndex] = remainder;
        } else {
          aryBuffer[driveIndex] = 0;
        }
    }
    if (driveIndex==1) {
        noteLengthInMs = beat / aryDuration1[aryNoteIndex[driveIndex]];
        noteLengthInMs = noteLengthInMs * 1000;
        freqOfSteps = 1000 / aryNotes1[aryNoteIndex[driveIndex]];
        freqOfSteps = freqOfSteps * 1000;
        aryNoteLength[driveIndex] = noteLengthInMs / freqOfSteps;
        aryNoteDelay[driveIndex] = freqOfSteps / 2;

        remainder = noteLengthInMs - (aryNoteLength[driveIndex] * freqOfSteps);
        if(remainder > 0) {
          scew = remainder % 100;
          remainder = remainder - scew;
          aryRunningScew[driveIndex] += scew;
          
          if(aryRunningScew[driveIndex]>=100) {
            remainder += 100;
            aryRunningScew[driveIndex] -= 100;
          }
          
          aryBuffer[driveIndex] = remainder;
        } else {
          aryBuffer[driveIndex] = 0;
        }
    }
    if (driveIndex==2) {
        noteLengthInMs = beat / aryDuration2[aryNoteIndex[driveIndex]];
        noteLengthInMs = noteLengthInMs * 1000;
        freqOfSteps = 1000 / aryNotes2[aryNoteIndex[driveIndex]];
        freqOfSteps = freqOfSteps * 1000;
        aryNoteLength[driveIndex] = noteLengthInMs / freqOfSteps;
        aryNoteDelay[driveIndex] = freqOfSteps / 2;

        remainder = noteLengthInMs - (aryNoteLength[driveIndex] * freqOfSteps);
        if(remainder > 0) {
          scew = remainder % 100;
          remainder = remainder - scew;
          aryRunningScew[driveIndex] += scew;
          
          if(aryRunningScew[driveIndex]>=100) {
            remainder += 100;
            aryRunningScew[driveIndex] -= 100;
          }
          
          aryBuffer[driveIndex] = remainder;
        } else {
          aryBuffer[driveIndex] = 0;
        }
    }
    aryNoteIndex[driveIndex]++;
    aryHighLow[driveIndex] = 0;
    aryWaiting[driveIndex] = 0;
  }else{
    aryHighLow[driveIndex]=4; // To stop the motor loop
  }
}

void setup() {
  //Serial.begin(9600);
  //while(true);
  //delay(500);
  for(driveIndex=driveStart;driveIndex<driveCount;driveIndex++) { // Initialise, and center the motor's of all drives
    pinMode(aryPinDirection[driveIndex], OUTPUT); 
    pinMode(aryPinHeadStep[driveIndex], OUTPUT);
    centerHead();
    setupNote();
  }
  //while(true);
  delay(1000);
  Timer1.initialize(timerDelay); // 1ms
  Timer1.attachInterrupt(play);
  
}

void play() {
  for(driveIndex=driveStart;driveIndex<driveCount;driveIndex++) {
    if(aryWaiting[driveIndex]<=0) { // If 0 (start of note)
      if(aryHighLow[driveIndex]==0) {
        digitalWrite(aryPinDirection[driveIndex], HIGH);
        digitalWrite(aryPinHeadStep[driveIndex],LOW);
        digitalWrite(aryPinHeadStep[driveIndex],HIGH);
        aryHighLow[driveIndex]=1;
        aryWaiting[driveIndex] = aryNoteDelay[driveIndex];
      }else if(aryHighLow[driveIndex]==1){
        digitalWrite(aryPinDirection[driveIndex], LOW);
        digitalWrite(aryPinHeadStep[driveIndex],LOW);
        digitalWrite(aryPinHeadStep[driveIndex],HIGH);
        aryHighLow[driveIndex]=2;
        aryWaiting[driveIndex] = aryNoteDelay[driveIndex];
      }else if(aryHighLow[driveIndex]==2) { // End of the note!
        aryNoteLength[driveIndex]--;
        if(aryNoteLength[driveIndex]==0) {
          if(aryBuffer[driveIndex]>0) {
            aryWaiting[driveIndex] = aryBuffer[driveIndex];
            aryHighLow[driveIndex] = 3;
          } else {
            setupNote();
          }
        } else {
          aryHighLow[driveIndex]=0;
        }
      }else if(aryHighLow[driveIndex]==3) {
        if(aryWaiting[driveIndex]<=0) {
          setupNote();
        }
      }
    }else{
      aryWaiting[driveIndex]=aryWaiting[driveIndex]-timeMinus;
    }
  }
}

void loop() {

}
